package as.adamsmith.renderscriptaudiodemo;

import android.app.Activity;
import android.media.AudioFormat;
import android.media.AudioManager;
import android.media.AudioTrack;
import android.os.Bundle;
import android.renderscript.Allocation;
import android.renderscript.Element;
import android.renderscript.RenderScript;
import android.widget.SeekBar;
import android.widget.SeekBar.OnSeekBarChangeListener;
import android.os.Process;

public class RSAudioDemoActivity extends Activity implements Runnable, OnSeekBarChangeListener {
	
	private static final int BLOCK_FRAMES = 512;
	private static final int SAMPLE_RATE_IN_HZ = 44100;
	private static final int CHANNEL_CONFIG = AudioFormat.CHANNEL_OUT_MONO;
	private static final int AUDIO_FORMAT = AudioFormat.ENCODING_PCM_16BIT;
	
	private final AudioTrack audioTrack = buildAudioTrack();
	
	private Thread thread;
	private RenderScript renderScript;
	private ScriptC_synth synth;
	
	private float energy = 0.0f;
	
	
	private AudioTrack buildAudioTrack() {
		int bufferSize = AudioTrack.getMinBufferSize(SAMPLE_RATE_IN_HZ, CHANNEL_CONFIG, AUDIO_FORMAT);
		AudioTrack audioTrack = new AudioTrack(
				AudioManager.STREAM_MUSIC,
				SAMPLE_RATE_IN_HZ,
				CHANNEL_CONFIG,
				AUDIO_FORMAT, bufferSize, AudioTrack.MODE_STREAM);
		return audioTrack;
	}

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
		renderScript = RenderScript.create(RSAudioDemoActivity.this);
		synth = new ScriptC_synth(renderScript, getResources(), R.raw.synth);
		((SeekBar)findViewById(R.id.seek)).setOnSeekBarChangeListener(this);
    }

    
    @Override
    public void onResume() {
    	super.onResume();
    	thread = new Thread(this, "renderscript synthesis driver");
    	thread.start();
    	audioTrack.play();
    }
    
    @Override
    public void onPause() {
    	super.onPause();
    	thread.interrupt();
    	try {
			thread.join();
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
    	thread = null;
    	audioTrack.stop();
    }
    
	@Override
	public void run() {
		
		Process.setThreadPriority(Process.THREAD_PRIORITY_AUDIO);
		
		short samples[] = new short[BLOCK_FRAMES];
		Allocation out = Allocation.createSized(renderScript, Element.I16(renderScript), samples.length, Allocation.USAGE_SCRIPT);
		
		synth.set_block_frames(BLOCK_FRAMES);
		synth.bind_block_out(out);
		synth.set_sample_rate(SAMPLE_RATE_IN_HZ);
		
		while(!Thread.interrupted()) {
			synth.set_energy(energy);
			synth.invoke_render_block();
			out.copyTo(samples);
			audioTrack.write(samples, 0, samples.length);
		}
	}
	
	@Override
	public void onProgressChanged(SeekBar seekBar, int progress,
			boolean fromUser) {
		energy = (float)progress/seekBar.getMax();
	}

	@Override
	public void onStartTrackingTouch(SeekBar seekBar) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void onStopTrackingTouch(SeekBar seekBar) {
		// TODO Auto-generated method stub
	}
    
}
